/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include "phone.h"
#include "phone-text.h"

static const char *basic_str[] = {
	N_("IDS_KPD_TAB4_KEYPAD"),
	N_("IDS_PB_ITAB4_FAVOURITES"),
	N_("IDS_PB_HEADER_SPEEDDIAL"),
	N_("IDS_CST_BODY_VOICE_MAIL"),
	N_("IDS_CLOG_ITAB4_LOGS"),
	N_("IDS_COM_POP_NO_SPEED_DIAL_NUMBER_ASSIGN_ONE_NOW_Q"),
	N_("IDS_PB_POP_REQUEST_FAILED"),
	N_("IDS_PB_POP_REQUEST_SUCCESS"),
	N_("IDS_CALL_POP_EMERGENCY_CALL"),
	N_("IDS_CLOG_BODY_NO_LOGS"),
	N_("IDS_PB_BODY_NO_FAVOURITES"),
	N_("IDS_CLOG_POP_MESSAGE_DELETED"),
	N_("IDS_PB_POP_MAXIMUM_PHONE_NUMBER_LENGTH_EXCEEDED"),
	N_("IDS_CLOG_POP_DELETING_LOGS_ING"),
	N_("IDS_PB_POP_DELETING_FAVOURITES_ING"),
	N_("IDS_PB_POP_INITIALISING_TRY_LATER_ING"),
	N_("IDS_COM_POP_SOUND_OFF"),
	N_("IDS_COM_POP_SOUND_ON"),
	N_("IDS_KPD_BODY_PREVIOUS_SCREEN_ABB"),
	N_("IDS_COM_BODY_BACK_TO_CALL"),
	N_("IDS_ST_BODY_INCOMMING_CALL"),
	N_("IDS_ST_BODY_OUTGOING_CALL"),
	N_("IDS_CLOG_HEADER_MISSED_CALL"),
	N_("IDS_CLOG_OPT_SENT_MESSAGE"),
	N_("IDS_CLOG_OPT_RECEIVED_MESSAGE"),
	N_("IDS_LOGS_BODY_REJECTED_CALL_ABB"),
	N_("IDS_LOGS_BODY_BLOCKED_CALL_ABB"),
	N_("IDS_CLOG_OPT_EDIT_NUMBER_BEFORE_CALL"),
};

static const char *base_op_str[] = {
	N_("IDS_CLOG_HEADER_SELECT_LOGS"),
	N_("IDS_CST_HEADER_SELECT_SPEED_DIAL_CONTACTS"),
	N_("IDS_CLOG_BUTTON2_UNBLOCK"),
	N_("IDS_CST_OPT_CHANGE_CONTACT_ABB"),
	N_("IDS_KPD_OPT_ADD_PD_SEC_PAUSE"),
	N_("IDS_KPD_OPT_ADD_WAIT"),
	N_("IDS_CLOG_HEADER_MISSED"),
	N_("IDS_COM_OPT_EXIT"),
};

static const char *number_type_str[] = {
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),/*Custom*/
	N_("IDS_PB_BODY_MOBILE"),/*Mobile*/
	N_("IDS_PB_BODY_HOME"),/*Home*/
	N_("IDS_PB_BODY_WORK"),/*Work*/
	N_("IDS_PB_BODY_FAX_HHOME"),/*Fax(home)*/
	N_("IDS_PB_BODY_FAX_HWORK"),/*Fax(work)*/
	N_("IDS_PB_BODY_ASSISTANT_S_NUMBER"),/*Assistant*/
	N_("IDS_PB_BODY_PAGER"),/*Pager*/
	N_("IDS_PB_BODY_OTHER"),/*Other*/
	N_("IDS_PB_BODY_TELEPHONE"),/*Telephone*/
	N_("IDS_PB_BODY_FAX"),/*Fax*/
	N_("IDS_PB_BODY_CAR_TELEPHONE"),/*Car*/
	N_("IDS_PB_BODY_HOME_2"),/*Home2*/
	N_("IDS_PB_BODY_WORK"),/*Work2*/ // TODO: Fixme
	N_("IDS_PB_BODY_COMPANY_MAIN"),/*Company main*/
	N_("IDS_PB_BODY_RADIO"),/*Radio*/
};

static const char *postal_type_str[] = {
	N_("IDS_PB_BODY_OTHER"),/*Other*/
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),/*Custom*/
	N_("IDS_PB_BODY_HOME"),/*Home*/
	N_("IDS_PB_BODY_WORK"),/*Work*/
};

static const char *cbasic_str[] = {
	N_("IDS_PB_BODY_ALL_CONTACTS"),/*All Contacts*/
	N_("IDS_PB_BODY_NOT_ASSIGNED"),/*Not Assigned*/
	N_("IDS_PB_BODY_SERVICE_NUMBER"),/*Service Number*/
	N_("IDS_PB_BODY_PHONE"),/*Phone*/
	N_("IDS_PB_BODY_GROUPS"),/*Groups*/
	N_("IDS_PB_BODY_DEFAULT"),/*Default*/
	N_("IDS_PB_BODY_RINGTONE"),/*Ringtone*/
	N_("IDS_PB_BODY_URL"),/*URL*/
	N_("IDS_PB_BODY_IM"),/*Instant Messenger*/
	N_("IDS_PB_BODY_EMAIL"),/*Email*/
	N_("IDS_PB_BODY_NUMBER"),/*Number*/
	N_("IDS_PB_BODY_NOTE"),/*Note*/
	N_("IDS_PB_BODY_POSTAL_ADDRESS"),/*Postal Address*/
	N_("IDS_PB_HEADER_DETAILS"),/*Details*/
	N_("IDS_PB_BODY_COMPANY"),/*Company*/
	N_("IDS_PB_BODY_ORDER_OF_DISPLAY_NAME"),/* Order display name */
	N_("IDS_PB_BODY_OWN_NUMBER"),/* Own number */
	N_("IDS_PB_BODY_FIRST_NAME"),
	N_("IDS_PB_BODY_LAST_NAME"),
	N_("IDS_PB_BODY_MIDDLE_NAME"),
	N_("IDS_PB_BODY_SUFFIX"),
	N_("IDS_PB_BODY_NICKNAME"),
	N_("IDS_PB_BODY_EXPORT_CONTACTS"),
	N_("IDS_PB_HEADER_SELECT_CONTACTS"),
	N_("IDS_PB_BODY_POSTAL_CODE"),
	N_("IDS_PB_BODY_COUNTRY"),
	N_("IDS_PB_BODY_PROVINCE"),
	N_("IDS_PB_BODY_CITY"),
	N_("IDS_PB_BODY_STREET"),
	N_("IDS_EMAIL_BODY_SAVED_IN_PS"),
	N_("IDS_PB_BODY_FREQUENTLY_CONTACTED_ABB"),
	N_("IDS_PB_POP_COPIED"),
	N_("IDS_COM_BODY_EXPORTED"),
	N_("IDS_PB_POP_DELETING_CONTACTS_ING"),
	N_("IDS_PB_BODY_MOVING_ING"),
	N_("IDS_PB_BODY_COPYING_ING"),
	N_("IDS_PB_POP_IMPORTING_SIM_CONTACTS_ING"),
	N_("IDS_PB_POP_IMPORTING_CONTACTS_FROM_VCARD_ING"),
	N_("IDS_PB_POP_EXPORTING_CONTACTS_TO_PS_ING"),
	N_("IDS_PB_BODY_VCARD"),
	N_("IDS_PB_BODY_READING_VCARD_ING"),
	N_("IDS_COM_BODY_IMPORTED_ABB"),
	N_("IDS_COM_BODY_NA"),
	N_("IDS_PB_OPT_ALL_LINKED_CONTACTS_ABB"),
	N_("IDS_PB_BODY_LINKED_CONTACTS"),
	N_("IDS_KA_BODY_MORE_INFORMATION"),
	N_("IDS_PB_BODY_JOB_TITLE"),
	N_("IDS_PB_BODY_DEPARTMENT"),
	N_("IDS_PB_OPT_GROUP_NAME"),
	N_("IDS_PB_BODY_ADD_ACCOUNT"),
	N_("IDS_PB_POP_ADDING_MEMBERS_TO_GROUP_ING"),
	N_("IDS_PB_OPT_VIEW"),
	N_("IDS_PB_BODY_PD_CONTACTS"),
	N_("IDS_PB_OPT_RENAME"),
	N_("IDS_PB_OPT_RELATIONSHIP"),
	N_("IDS_PB_BODY_NO_HISTORIES"),
	N_("IDS_PB_BODY_NO_ACTIVITIES"),
	N_("IDS_PB_POP_ADDING_ING"),
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),
	N_("IDS_PB_BODY_EDITADDITIONALINFO"),
	N_("IDS_PB_BODY_PO_BOX"),
	N_("IDS_PB_BODY_STATE"),
	N_("IDS_PB_BODY_DEFAULT_GROUP_FRIENDS"),
	N_("IDS_PB_BODY_DEFAULT_GROUP_FAMILY"),
	N_("IDS_PB_BODY_CO_WORKERS"),
	N_("IDS_PB_BODY_DEFAULT_GROUP_WORK"),
	N_("IDS_PB_BODY_VIBRATION_PATTERN"),
	N_("IDS_PB_OPT_SET_DEFAULT"),
	N_("IDS_COM_OPT_EVENT"),
	N_("IDS_PB_OPT_SHARE_CONTACT_INFORMATION"),
	N_("IDS_PB_BODY_1_CONTACT"),
	N_("IDS_COM_OPT1_WI_FI_DIRECT"),
	N_("IDS_PB_OPT_EXPORT_TO_SIM_CARD"),
	N_("IDS_PB_SK3_INFO"),
	N_("IDS_PB_TAB4_HISTORY"),
	N_("IDS_PB_TAB4_ACTIVITIES"),
	N_("IDS_CMT_HEADER_GROUP_MESSAGE_TMO"),
	N_("IDS_CMT_HEADER_GROUP_EMAIL_TMO"),
};

static const char *err_str[] = {
	N_("IDS_PB_BODY_NO_NAME"),
	N_("IDS_PB_POP_INVALID_EMAIL_ADDRESS"),
	N_("IDS_PB_POP_COMPLETE_AT_LEAST_ONE_FIELD_AS_WELL_AS_NAME"),
	N_("IDS_IM_BODY_FAILED_TO_ACCESS_DATABASE"),
	N_("IDS_PB_POP_ALREADY_ADDED"),
	N_("IDS_PB_POP_GROUP_ALREADY_IN_USE"),
	N_("IDS_PB_BODY_NO_GROUP"),
	N_("IDS_PB_POP_NO_NUMBER"),
	N_("IDS_PB_POP_NO_EMAIL_ADDRESSES"),
	N_("IDS_PB_BODY_NO_CONTACTS"),
	N_("IDS_PB_POP_SIM_MEMORY_EMPTY"),
	N_("IDS_PB_POP_MAXIMUM_NUMBER_OF_ENTRIES_EXCEEDED"),
	N_("IDS_SEARCH_BODY_DATA_DOES_NOT_EXIST"),
	N_("IDS_PB_BODY_INVALID_NUMBER"),
	N_("IDS_PB_POP_SIM_MEMORY_FULL"),
	N_("IDS_PB_BODY_YOU_CAN_IMPORT_CONTACTS_FROM_SIM_CARD_TO_YOUR_PHONE_IN_SETTINGS_HELP_MSG"),
	N_("IDS_CLOG_POP_NO_LOGS_AFTER_YOU_MAKE_RECEIVE_A_CALL_OR_SEND_RECEIVE_A_MESSAGE_LOGS_WILL_BE_SHOWN"),
	N_("IDS_PB_POP_UNABLE_TO_SEND_NAME_CARDS"),
};

static const char *cbase_op_str[] = {
	N_("IDS_PB_BODY_MARK_AS_DEFAULT_NUMBER"),/*Set Default Number*/
	N_("IDS_PB_BODY_MARK_AS_DEFAULT_EMAIL"),/*Set Default Email*/
	N_("IDS_PB_BODY_IMAGE"),/*Image*/
	N_("IDS_PB_BODY_TAKE_A_PHOTO"),/*Take a photo*/
	N_("IDS_PB_OPT_REMOVE_PHOTO_ABB"),/*Delete photo*/
	N_("IDS_PB_SK1_ADD_TO_HOME_SCREEN"),/*Add to Home*/
	N_("IDS_PB_BODY_SAVE_CONTACT_TO"),/*Save contact to*/
	N_("IDS_PB_OPT_SAVE_TO"),/*Save to*/
	N_("IDS_PB_BODY_IMPORT_SIM_CONTACTS"), /* Import SIM contacts */
	N_("IDS_PB_BODY_FIRST_NAME_FIRST"),/* order display - First name first */
	N_("IDS_PB_BODY_LAST_NAME_FIRST_ABB"),/* order display - Last name first */
	N_("IDS_ST_SK_SHOW"),/* setting - mfc show */
	N_("IDS_ST_SK_HIDE"),/* setting - mfc hide */
	N_("IDS_PB_HEADER_SEND_NAMECARD"),
	N_("IDS_PB_HEADER_CREATE_GROUP"),
	N_("IDS_PB_HEADER_SELECT_GROUP"),
	N_("IDS_PB_BODY_EXPORT_CONTACTS"),
	N_("IDS_PB_SK_IMPORT_CONTACTS"),
	N_("IDS_PB_BODY_LINK_CONTACT"),
	N_("IDS_PB_OPT_UNLINK"),
	N_("IDS_PB_POP_ALL_LINKED_CONTACTS_WILL_BE_DELETED"),
	N_("IDS_PB_BODY_ADD_TO_FAVOURITES"),
	N_("IDS_PB_HEADER_EDIT_GROUP"),
	N_("IDS_PB_SK_ADD_MEMBER"),
};

static const char *event_type_str[] = {
	N_("IDS_PB_HEADER_UNKNOWN"),/*CTS_EVENT_TYPE_OTHER*/
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),/*CTS_EVENT_TYPE_CUSTOM*/
	N_("IDS_PB_BODY_BIRTHDAY"),/*CTS_EVENT_TYPE_BIRTH*/
	N_("IDS_PB_BODY_ANNIVERSARY")/*CTS_EVENT_TYPE_ANNIVERSARY*/
};

static const char *im_type_str[] = {
	N_("IDS_PB_BODY_OTHER"),/*CTS_IM_TYPE_OTHER*/
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),/*CTS_IM_TYPE_CUSTOM*/
	N_("IDS_PB_OPT_GOOGLE_TALK"),/*CTS_IM_TYPE_GOOGLE*/
	N_("IDS_PB_OPT_WINDOWS_LIVE_MESSENGER"),/*CTS_IM_TYPE_WLM*/
	N_("IDS_PB_BODY_YAHOO"),/*CTS_IM_TYPE_YAHOO*/
	N_("IDS_PB_BODY_FACEBOOK"),/*CTS_IM_TYPE_FACEBOOK*/
	N_("IDS_PB_BODY_IMPSCOMMUNITY_ICQ"),/*CTS_IM_TYPE_ICQ*/
	N_("IDS_PB_BODY_IMPSCOMMUNITY_AIM"),/*CTS_IM_TYPE_AIM*/
	N_("IDS_PB_OPT_QQ"),/*CTS_IM_TYPE_QQ*/
	N_("IDS_PB_OPT_JABBER"),/*CTS_IM_TYPE_JABBER*/
	N_("IDS_PB_OPT_SKYPE"),/*CTS_IM_TYPE_SKYPE*/
};

static const char *day_str[] = {
	N_("IDS_COM_BODY_SUNDAY"),
	N_("IDS_COM_BODY_MONDAY"),
	N_("IDS_COM_BODY_TUESDAY"),
	N_("IDS_COM_BODY_WEDNESDAY"),
	N_("IDS_COM_BODY_THURSDAY"),
	N_("IDS_COM_BODY_FRIDAY"),
	N_("IDS_COM_BODY_SATURDAY"),
};

static const char* relation_type_str[] = {
	N_("IDS_PB_BODY_OTHER"),/*None*/
	N_("IDS_PB_BUTTON_ASSISTANT"),/*Assistant*/
	N_("IDS_COM_OPT_BROTHER_ABB"),/*Brother*/
	N_("IDS_PB_BUTTON_CHILD"),/*Child*/
	N_("IDS_PB_OPT_DOMESTIC_PARTNER_ABB"),/*Domestic partner*/
	N_("IDS_COM_OPT_FATHER_ABB"),/*Father*/
	N_("IDS_PB_BODY_CALLERIDFRIEND"),/*Friend*/
	N_("IDS_JAVA_BODY_MANAGER"),/*Manager*/
	N_("IDS_COM_OPT_MOTHER_ABB"),/*Mother*/
	N_("IDS_COM_OPT_PARENT_ABB"),/*Parent*/
	N_("IDS_PB_BODY_CALLERIDPARTNER"),/*Partner*/
	N_("IDS_PB_OPT_REFERRED_BY"),/*Referred by*/
	N_("IDS_COM_BODY_RELATIVE_ABB"),/*Relative*/
	N_("IDS_COM_OPT_SISTER_ABB"),/*Sister*/
	N_("IDS_PB_BODY_SPOUSE"),/*Spouse*/
	N_("IDS_PB_BODY_CUSTOM_T_PHONEBOOK"),/*Custom*/
};

static const char *month_str[] = {
	"IDS_COM_BODY_NONE",
	"IDS_COM_BODY_JAN",
	"IDS_COM_BODY_FEB",
	"IDS_COM_BODY_MAR",
	"IDS_COM_BODY_APR",
	"IDS_COM_BODY_MAY",
	"IDS_COM_BODY_JUN",
	"IDS_COM_BODY_JUL",
	"IDS_COM_BODY_AUG",
	"IDS_COM_BODY_SEP",
	"IDS_COM_BODY_OCT",
	"IDS_COM_BODY_NOV",
	"IDS_COM_BODY_DEC"
};

const char* phone_get_text(int op, int type)
{
	const char **table;

	switch (op) {
	case PH_GET_TEXT_BASIC:
		table = basic_str;
		break;
	case PH_GET_TEXT_BASE_OP:
		table = base_op_str;
		break;
	case CT_GET_TEXT_BASIC:
		table = cbasic_str;
		break;
	case CT_GET_TEXT_ERR:
		table = err_str;
		break;
	case CT_GET_TEXT_BASE_OP:
		table = cbase_op_str;
		break;
	case CT_GET_TEXT_NUM_TYPE:
		table = number_type_str;
		break;
	case CT_GET_TEXT_POSTAL_TYPE:
	case CT_GET_TEXT_WEBSITE_TYPE:
	case CT_GET_TEXT_EMAIL_TYPE:
		table = postal_type_str;
		break;
	case CT_GET_TEXT_EVENT_TYPE:
		table = event_type_str;
		break;
	case CT_GET_TEXT_IM_TYPE:
		table = im_type_str;
		break;
	case CT_GET_TEXT_MONTH:
		if (sizeof(month_str) / sizeof(month_str[0]) <= type)
			return NULL;
		else
			return dgettext("sys_string", month_str[type]);
	case CT_GET_TEXT_RELATIONSHIP_TYPE:
		table = relation_type_str;
		break;
	case PH_GET_TEXT_DAY:
		if (sizeof(day_str) / sizeof(day_str[0]) <= type)
			return NULL;
		else
			table = day_str;
		break;
	default:
		return NULL;
	}
	return dgettext(PACKAGE, table[type]);
}

static const char *sys_str[] = {
	"IDS_COM_SK_OK",
	"IDS_COM_SK_SAVE",
	"IDS_COM_SK_DONE",
	"IDS_COM_SK_EDIT",
	"IDS_COM_SK_DELETE",
	"IDS_COM_SK_CANCEL",
	"IDS_COM_SK3_BLOCK",
	"IDS_COM_SK_NO",
	"IDS_COM_SK_YES",
	"IDS_COM_SK_ADD",
	"IDS_COM_SK_MORE",
	"IDS_COM_SK_NEXT",
	"IDS_COM_BODY_CALL",
	"IDS_COM_BODY_MESSAGE",
	"IDS_COM_BODY_VIDEO_CALL",
	"IDS_COM_BODY_UNKNOWN",
	"IDS_COM_BODY_SELECT_ALL",
	"IDS_COM_BODY_CONTACTS",
	"IDS_COM_BODY_DETAILS",
	"IDS_COM_BODY_REMOVE",
	"IDS_COM_BODY_TODAY",
	"IDS_COM_BODY_YESTERDAY",
	"IDS_COM_BODY_PREVIOUS",
	"IDS_COM_BODY_NO_NAME",
	"IDS_COM_BODY_CREATE",
	"IDS_COM_BODY_MOVE",
	"IDS_COM_BODY_COPY",
	"IDS_COM_BODY_PHONE",
	"IDS_COM_BODY_SEARCH",
	"IDS_COM_BODY_BLUETOOTH",
	"IDS_COM_BODY_EMAIL",
	"IDS_COM_BODY_ADDTOPHONEBOOK_SIM",
	"IDS_COM_BODY_MEMORY_CARD",
	"IDS_COM_BODY_DETAILS_NAME",
	"IDS_COM_BODY_ALL",
	"IDS_COM_POP_FAIL",
	"IDS_COM_POP_FAILED",
	"IDS_COM_POP_CLOSE",
	"IDS_COM_POP_REMOVED",
	"IDS_COM_POP_DELETED",
	"IDS_COM_POP_INCORRECT_PASSWORD",
	"IDS_COM_POP_ALREDY_EXISTS",
	"IDS_COM_POP_SELECTED",
	"IDS_COM_POP_SUCCESS",
	"IDS_COM_POP_ERROR",
	"IDS_COM_POP_MOVED",
	"IDS_COM_POP_ADDED",
	"IDS_COM_POP_NOT_ENOUGH_MEMORY",
	"IDS_COM_POP_NOTHING_SELECTED",
	"IDS_COM_POP_FILE_NOT_FOUND",
	"IDS_COM_POP_ADDRESS",
	"IDS_COM_OPT_ADD_TO_CONTACTS",
	"IDS_COM_BODY_SELECTED_DATA_NOT_FOUND",
	"IDS_COM_POP_DELETE_Q",
	"IDS_COM_SK_SET",
	"IDS_COM_BODY_RESET",
};

const char* phone_get_system_string(int type)
{
	return dgettext("sys_string", sys_str[type]);
}
