/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/



#include <ui-gadget-module.h>
#include <vconf.h>
#include <vconf-keys.h>
#include <fcntl.h>

#include "phone.h"
#include "ct-detail.h"
#include "ct-detail-utils.h"
#include "ct-detail-view.h"
#include "ct-input-view.h"
#include "ct-input-utils.h"

static int create_ug_detail_view(Evas_Object *base, int request, ct_detail_data *cdetail_d)
{
	p_retvm_if(NULL == base, FALSE, "base is NULL");

	cdetail_d->navi = elm_naviframe_add(base);
	p_retvm_if (NULL == cdetail_d->navi, FALSE, "elm_naviframe_add() return NULL");

	elm_naviframe_prev_btn_auto_pushed_set(cdetail_d->navi, EINA_FALSE);
	evas_object_show(cdetail_d->navi);
	elm_object_part_content_set(base, "elm.swallow.content", cdetail_d->navi);

	cdetail_d->ug_detail = true;
	if (CT_UG_REQUEST_DETAIL == request) {
		Evas_Object *content;

		content = ct_detail_create_content(cdetail_d);
		if (NULL == content)
			return FALSE;

		cdetail_d->navi_item = elm_naviframe_item_push(cdetail_d->navi, NULL, NULL, NULL, content, NULL);
		elm_naviframe_item_title_visible_set(cdetail_d->navi_item, EINA_FALSE);
		ct_detail_set_navi(cdetail_d, DETAIL_VIEW_MODE_INFO);
	}
	else {
		int contact_id = 0;
		contacts_record_h record_contact = cdetail_d->contact;
		if (record_contact)
			contacts_record_get_int(record_contact, _contacts_contact.id, &contact_id);

		if (NULL == cdetail_d->list_contacts)
			cdetail_d->list_contacts = eina_list_append(cdetail_d->list_contacts, cdetail_d->contact);

		ct_input_push_navi_content(cdetail_d, record_contact, contact_id);
	}
	return TRUE;
}

static int __detail_set_contact_data(contacts_record_h contact, char *data_type, char *value)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;
	PH_DBG("data_type %s value %s", data_type, value);

	if(strcmp(CT_OPERATION_PHONE, data_type) == 0) {
		err = contacts_record_create(_contacts_number._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_int(record, _contacts_number.type, CONTACTS_NUMBER_TYPE_HOME | CONTACTS_NUMBER_TYPE_CELL);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_number.number, value);
		contacts_record_add_child_record(contact, _contacts_contact.number, record);
	}
	else if(strcmp(CT_OPERATION_EMAIL, data_type) == 0) {
		err = contacts_record_create(_contacts_email._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_int(record, _contacts_email.type, CONTACTS_EMAIL_TYPE_HOME);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_email.email, value);
		contacts_record_add_child_record(contact, _contacts_contact.email, record);
	}
	else if(strcmp(CT_OPERATION_URL, data_type) == 0) {
		err = contacts_record_create(_contacts_url._uri, &record);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_create() Failed(%d)", err);

		err = contacts_record_set_int(record, _contacts_url.type, CONTACTS_URL_TYPE_HOME);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_int() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_url.url, value);
		contacts_record_add_child_record(contact, _contacts_contact.url, record);
	}

	return 0;
}


static int __detail_parse_bundle_by_operation(service_h service, char* operation, ct_detail_data *cdetail_d)
{
	char* val = NULL;
	char* val1 = NULL;
	contacts_error_e err = CONTACTS_ERROR_NONE;

	PH_DBG("operaion = %s", operation);
	cdetail_d->service_operation = service;

	if(strcmp(operation, CT_OPERATION_SOCIAL_VIEW) == 0) {
		service_get_extra_data(service, CT_OPERATION_ITEM_TYPE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_ITEM_TYPE_PERSON) != 0) {
				ERR("CT_OPERATION_ITEM_TYPE_PERSON is not matched %s", val);
				free(val);
				return -1;
			}
			free(val);
			val = NULL;

			service_get_extra_data(service, CT_OPERATION_ITEM_ID, &val1);
			if(val1) {
				cdetail_d->person_id = atoi(val1);
				free(val1);
				return CT_UG_REQUEST_DETAIL;
			}
			else {
				ERR("CT_OPERATION_ITEM_ID is mandatory");
				return -1;
			}

		}
		else {
			ERR("CT_OPERATION_ITEM_TYPE is mandatory");
			return -1;
		}
	}
	else if(strcmp(operation, CT_OPERATION_SOCIAL_ADD) == 0) {
		service_get_extra_data(service, CT_OPERATION_ITEM_TYPE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_ITEM_TYPE_CONTACT) != 0) {
				ERR("CT_OPERATION_ITEM_TYPE_CONTACT is not matched %s", val);
				free(val);
				return -1;
			}
			free(val);
			val = NULL;
		}
		else {
			ERR("CT_OPERATION_ITEM_TYPE is mandatory");
			return -1;
		}

		service_get_extra_data(service, CT_OPERATION_PHONE, &val1);
		if(val1)
		{
			if (NULL == cdetail_d->contact) {
				err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
				if(CONTACTS_ERROR_NONE != err) {
					ERR("contacts_record_create failed %s", err);
					return -1;
				}
			}

			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_PHONE, val1);

			free(val1);
			val1 = NULL;
		}

		service_get_extra_data(service, CT_OPERATION_EMAIL, &val1);
		if(val1)
		{
			if (NULL == cdetail_d->contact) {
				err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
				if(CONTACTS_ERROR_NONE != err) {
					ERR("contacts_record_create failed %s", err);
					return -1;
				}
			}

			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_EMAIL, val1);

			free(val1);
			val1 = NULL;
		}

		service_get_extra_data(service, CT_OPERATION_URL, &val1);
		if(val1)
		{
			if (NULL == cdetail_d->contact) {
				err = contacts_record_create(_contacts_contact._uri, &(cdetail_d->contact));
				if(CONTACTS_ERROR_NONE != err) {
					ERR("contacts_record_create failed %s", err);
					return -1;
				}
			}
			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_URL, val1);

			free(val1);
			val1 = NULL;
		}

		return CT_UG_REQUEST_ADD;
	}
	else if(strcmp(operation, CT_OPERATION_SOCIAL_EDIT) == 0) {
		service_get_extra_data(service, CT_OPERATION_ITEM_TYPE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_ITEM_TYPE_PERSON) != 0) {
				ERR("CT_OPERATION_ITEM_TYPE_CONTACT is not matched %s", val);
				free(val);
				return -1;
			}

			service_get_extra_data(service, CT_OPERATION_ITEM_ID, &val1);
			if(val1) {
				PH_DBG("person id %s", val1);
				cdetail_d->person_id = atoi(val1);
				free(val1);
				val1 = NULL;

				if (cdetail_d->person_id) { // From OSP
					Eina_List *list_contact = ctui_person_get_writable_contact_list(cdetail_d->person_id);
					if (list_contact) {
						cdetail_d->contact = list_contact->data;
						eina_list_free(list_contact);
					}
				}
			}
			else {
				ERR("CT_OPERATION_ITEM_ID is mandatory");
				free(val);
				return -1;
			}
			free(val);
			val = NULL;
		}
		else {
			ERR("CT_OPERATION_ITEM_TYPE is mandatory");
			return -1;
		}

		service_get_extra_data(service, CT_OPERATION_PHONE, &val1);
		if(val1)
		{
			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_PHONE, val1);

			free(val1);
			val1 = NULL;
		}

		service_get_extra_data(service, CT_OPERATION_EMAIL, &val1);
		if(val1)
		{
			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_EMAIL, val1);

			free(val1);
			val1 = NULL;
		}

		service_get_extra_data(service, CT_OPERATION_URL, &val1);
		if(val1)
		{
			__detail_set_contact_data(cdetail_d->contact, CT_OPERATION_URL, val1);

			free(val1);
			val1 = NULL;
		}
		return CT_UG_REQUEST_EDIT;
	}
	else {
		ERR("operation wrong");
		return -1;
	}
}

static void *detail_on_create(ui_gadget_h ug, enum ug_mode mode,
		service_h service, void *priv)
{
	int ret;
	int request = 0;
	Evas_Object *bg;
	Evas_Object *base;
	Evas_Object *parent;
	contacts_error_e err;
	ct_detail_data *cdetail_d = priv;
	char *operation = NULL;
	cdetail_d->base_ab = -1;

	p_retvm_if(NULL == ug || NULL == priv, NULL, "The parameter is invalid(ug=%p, priv=%p)", ug, priv);

	cdetail_d->win = ug_get_window();
	parent = ug_get_parent_layout(ug);
	p_retvm_if(NULL == parent, NULL, "ug_get_parent_layout() return NULL");
	cdetail_d->ug = ug;

	bindtextdomain(PACKAGE, LOCALEDIR);

	err = contacts_connect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_connect2() Failed(%d)", err);
	PH_DBG("contacts_connect()");

	service_get_operation(service, &operation);

	if(operation) {
		request = __detail_parse_bundle_by_operation(service, operation, cdetail_d);
		free(operation);
		if(request < 0) {
			ERR("__detail_parse_bundle_by_operation invalid parameter");
			request = ct_detail_parse_bundle(service, cdetail_d, &ret);
		}
	}
	else
		request = ct_detail_parse_bundle(service, cdetail_d, &ret);

	if (UG_MODE_FULLVIEW == mode)
		base = phone_create_base_layout(parent, true);
	else
		base = phone_create_base_layout(parent, false);


	if (-2 == ret) {
		Evas_Object *popup;
		popup = elm_popup_add(base);
		evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		elm_object_text_set(popup, S_(PH_SYS_BODY_SELECTED_DATA_NOT_FOUND));
		elm_popup_timeout_set(popup, 1.5);
		evas_object_show(popup);
		evas_object_smart_callback_add(popup, "timeout", ctui_list_ug_hide_cb, cdetail_d->ug);
		return base;
	}
	p_retvm_if(request < 0, NULL, "ct_detail_parse_bundle() Failed(%d)", request);

	bg = phone_create_bg(base);
	if (NULL == bg) {
		ERR("phone_create_bg() return NULL");
		evas_object_del(base);
		err = contacts_disconnect2();
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
		return NULL;
	}

	if (!create_ug_detail_view(base, request, cdetail_d)) {
		evas_object_del(base);
		err = contacts_disconnect2();
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
	}
	return base;
}

static void detail_on_destroy(ui_gadget_h ug, service_h service, void *priv)
{
	contacts_error_e err;
	ct_detail_data *cdetail_d = priv;
	p_retm_if(NULL == ug, "The parameter is invalid(ug=%p)", ug);

	if (cdetail_d && !cdetail_d->navi)
		free(cdetail_d);

	evas_object_del(ug_get_layout(ug));

	err = contacts_disconnect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
}

static void detail_on_key_event(ui_gadget_h ug, enum ug_key_event event, service_h service, void *priv)
{
	PH_FN_CALL;
	ct_detail_data *cdetail_d = priv;

	if (!ug)
		return;

	switch (event) {
	case UG_KEY_EVENT_END:
		if (cdetail_d->popup) {
			ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);
		}
		else {
			if (elm_naviframe_top_item_get(cdetail_d->navi)
					== cdetail_d->navi_item) {
				ug_destroy_me(ug);
			}
			else
				elm_naviframe_item_pop(cdetail_d->navi);
		}
		break;
	default:
		break;
	}
}

static void detail_on_message(ui_gadget_h ug, service_h msg, service_h service, void *priv)
{
	char* val = NULL;
	ct_detail_data *cdetail_d = priv;

	if (UG_MODE_FRAMEVIEW == ug_get_mode(ug)) {
		if (msg) {
			service_get_extra_data(service, "key_event", &val);
			if (val && CTUG_MSG_KEY_END == atoi(val)) {
				if (cdetail_d->popup) {
					ct_detail_popup_hide_cb(cdetail_d, NULL, NULL);
				}
				else {
					if (elm_naviframe_top_item_get(cdetail_d->navi)
							== cdetail_d->navi_item) {
						ug_destroy_me(ug);
					}
					else
						elm_naviframe_item_pop(cdetail_d->navi);
				}
			}
			free(val);
		}
	}
}

static void detail_on_pause(ui_gadget_h ug, service_h service, void *priv)
{
	contacts_name_display_order_e name_display_order;
	ct_detail_data *cdetail_d = priv;
	contacts_setting_get_name_display_order(&name_display_order);
	cdetail_d->order_display = (int)name_display_order;
}

static void detail_on_resume(ui_gadget_h ug, service_h service, void *priv)
{
	contacts_name_display_order_e order;
	ct_detail_data *cdetail_d = priv;

	contacts_setting_get_name_display_order(&order);
	if (cdetail_d->order_display != order)
		ctui_detail_genlist_update(cdetail_d);
}

static void detail_on_event(ui_gadget_h ug, enum ug_event event, service_h service, void *priv)
{
	ct_detail_data *cdetail_d = priv;
	switch (event) {
	case UG_EVENT_ROTATE_PORTRAIT:
	case UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN:
		break;
	case UG_EVENT_ROTATE_LANDSCAPE:
	case UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN:
		break;
	case UG_EVENT_LANG_CHANGE:
		ct_detail_view_language_changed(cdetail_d);
		break;
	default:
		break;
	}
}

API int UG_MODULE_INIT(struct ug_module_ops *ops)
{
	ct_detail_data *cdetail_d;

	p_retvm_if(!ops, -1, "ops is NULL");

	cdetail_d = calloc(1, sizeof(ct_detail_data));
	p_retvm_if(!cdetail_d, -1, "UG data is NULL, Not enough memory");

	ops->create = detail_on_create;
	ops->start = NULL;
	ops->pause = detail_on_pause;
	ops->resume = detail_on_resume;
	ops->destroy = detail_on_destroy;
	ops->message = detail_on_message;
	ops->event = detail_on_event;
	ops->key_event = detail_on_key_event;
	ops->priv = cdetail_d;
	ops->opt = UG_OPT_INDICATOR_ENABLE;

	return 0;
}

API void UG_MODULE_EXIT(struct ug_module_ops *ops)
{
	p_retm_if(!ops, "ops is NULL");

	ops->priv = NULL;
}

API int ctui_create_detail_view(ct_list_data *clist_d, ct_person_list *person_info)
{
	PH_FN_CALL;
	PH_START_TIME_CHECK;
	ct_detail_data *cdetail_d;
	Evas_Object *content;

	cdetail_d = calloc(1, sizeof(ct_detail_data));
	p_retvm_if(!cdetail_d, -1, "cdetail_d is NULL, Not enough memory");

	ctui_ld_to_dd(clist_d, cdetail_d);
	cdetail_d->person_id = person_info->id;
	cdetail_d->titem = NULL;
	cdetail_d->ctrl_button = NULL;
	content = ct_detail_create_content(cdetail_d);
	if (NULL == content) {
		ERR("ct_detail_create_content() Failed");
		free(cdetail_d);
		return -1;
	}

	cdetail_d->navi_item = elm_naviframe_item_push(cdetail_d->navi, NULL, NULL, NULL, content, NULL);
	elm_naviframe_item_title_visible_set(cdetail_d->navi_item, EINA_FALSE);
	ct_detail_set_navi(cdetail_d, DETAIL_VIEW_MODE_INFO);

	PH_DBG("##### [TSP_TEST] list->detail end#####");

	PH_END_TIME_CHECK();
	return 0;
}


API void ct_detail_view_language_changed(ct_detail_data *cdetail_d)
{
	PH_FN_CALL;
	p_retm_if(NULL == cdetail_d, "parameter(cdetail_d) is NULL");

	// 1. update title
	if (cdetail_d->navi_item)
		elm_object_item_text_set(cdetail_d->navi_item, T_(CT_GET_TEXT_BASIC, CTTEXT_DETAILS));

	// 2. update content
	ct_detail_refresh_view(cdetail_d);

	// 3. update controlbar


	if (cdetail_d->has_input)
		ct_input_language_changed(cdetail_d->navi);
}


API int ctui_create_input_view(ct_detail_data *cdetail_d)
{
	PH_FN_CALL;
	PH_START_TIME_CHECK;
	int contact_id = 0;
	char addressbook_name[50] = {0,};
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record_contact_clone = NULL;

	p_retvm_if(!cdetail_d, -1, "cdetail_d is NULL");
	if (cdetail_d->contact) {
		err = contacts_record_clone(cdetail_d->contact, &record_contact_clone);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_record_clone() Failed(%d)", err);
			return 0;
		}
		contacts_record_get_int(cdetail_d->contact, _contacts_contact.id, &contact_id );
	}
	else if (cdetail_d->person_id) {
		Eina_List *list_contact;
		list_contact = ctui_person_get_writable_contact_list(cdetail_d->person_id);
		if (NULL == list_contact){
			err = contacts_record_create(_contacts_contact._uri, &record_contact_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_record_create() Failed(%d)", err);
				return 0;
			}
		}
		else {
			err = contacts_record_clone(cdetail_d->contact, &record_contact_clone);
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_record_clone() Failed(%d)", err);
				return 0;
			}
			ctui_free_record_list(list_contact);
		}
	}

	ctui_get_addressbook_name(	ct_input_get_base_ab(), addressbook_name, sizeof(addressbook_name));
	PH_DBG("addressbook_name : %s",addressbook_name );

	ct_input_push_navi_content(cdetail_d, record_contact_clone, contact_id);

	PH_END_TIME_CHECK("ct_input_push_navi_content");
	return 0;
}

API Evas_Object* ctui_contacts_viewer(Evas_Object *win, Evas_Object *navi, int personid_vcardidx, const char *vcard_path)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_detail_data *cdetail_d;
	Evas_Object *bg;
	Evas_Object *base = NULL;
	Evas_Object *content;
	Evas_Object *conform = NULL;

	cdetail_d = calloc(1, sizeof(ct_detail_data));
	p_retvm_if(!cdetail_d, NULL, "cdetail_d is NULL, Not enough memory");

	cdetail_d->win = win;
	if (navi)
		cdetail_d->navi = navi;
	else {
		conform = elm_conformant_add(win);
		p_retvm_if(NULL == conform, NULL, "elm_conformant_add() Failed");
		evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

		base = phone_create_base_layout(conform, true);
		cdetail_d->navi = elm_naviframe_add(base);
		evas_object_show(cdetail_d->navi);

		bg = phone_create_bg(base);
		if (NULL == bg) {
			ERR("phone_create_bg() return NULL");
			evas_object_del(base);
			free(cdetail_d);
			return NULL;
		}
		elm_object_style_set(bg, "group_list");
	}

	evas_object_data_set(cdetail_d->win, "naviframe", (void *)cdetail_d->navi);

	if(NULL== vcard_path){
		int person_id = personid_vcardidx;
		if (person_id) {
			cdetail_d->is_shortcut = true;
			cdetail_d->person_id = person_id;
			err = contacts_db_get_record(_contacts_person._uri, person_id, &(cdetail_d->person));
			if (CONTACTS_ERROR_NONE != err) {
				ERR("contacts_db_get_record(%d) Failed(%d)", person_id, err);
				if (base)
					evas_object_del(base);
				free(cdetail_d);
				return NULL;
			}
			cdetail_d->list_contacts = ctui_person_get_contact_list(cdetail_d->person_id);
		}
		else
			ERR("personid_vcardidx <= 0");
	}
	else {
		int fd = 0;
		int size = 0;
		int vcard_idx = personid_vcardidx;
		char buf[CT_VCARD_FILE_MAX_SIZE+1] = {0, };

		contacts_record_h record;
		contacts_list_h list_contacts_vcard = NULL;
		cdetail_d->vcard_path = strdup(vcard_path);

		if (NULL == cdetail_d->vcard_path) {
			ERR("strdup() return NULL");
			if (base)
				evas_object_del(base);
			free(cdetail_d);
			return NULL;
		}

		fd = open(cdetail_d->vcard_path, O_RDONLY);

		p_retvm_if(-1 == fd, NULL, "open() return -1");
		size = read(fd, buf, sizeof(buf)-1);
		close(fd);

		err = contacts_vcard_parse_to_contacts(buf, &list_contacts_vcard);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_vcard_parse_to_contacts(%s) Failed(%d)", vcard_path, err);
			if (base)
				evas_object_del(base);
			free(cdetail_d);
			return NULL;
		}

		if ( vcard_idx <= 0 ) {
			err = contacts_list_get_current_record_p(list_contacts_vcard, &record);
			if (CONTACTS_ERROR_NONE == err) {
				err = contacts_record_clone(record, &(cdetail_d->contact));
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_clone() Failed(%d)", err);
			}
		}
		else{
			int idx = 0;
			err = CONTACTS_ERROR_NONE;
			idx=1;
			while (CONTACTS_ERROR_NONE == err) {
					if(vcard_idx == idx){
					err = contacts_list_get_current_record_p(list_contacts_vcard, &record);
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_get_current_record_p() Failed(%d)", err);
					err = contacts_record_clone(record, &(cdetail_d->contact));
					p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_clone() Failed(%d)", err);
					break;
				}
				err = contacts_list_next(list_contacts_vcard);
				if (CONTACTS_ERROR_NONE != err) {
					ERR("contacts_list_next() Failed(%d) vcard_idx %d, idx %d", err, vcard_idx, idx);
					break;
				}
				++idx;
			}
		}
		err = contacts_list_destroy(list_contacts_vcard, true);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_list_destroy(list_contacts_vcard) Failed(%d)", err);
	}

	content = ct_detail_create_content(cdetail_d);
	if (NULL == content) {
		ERR("ct_detail_create_content() Failed");
		if (base)
			evas_object_del(base);
		if (cdetail_d->contact)
			contacts_record_destroy(cdetail_d->contact, true);
		if (cdetail_d->person)
			contacts_record_destroy(cdetail_d->person, true);
		free(cdetail_d);
		return NULL;
	}

	cdetail_d->navi_item = elm_naviframe_item_push(cdetail_d->navi,NULL, NULL, NULL, content, NULL);
	elm_naviframe_item_title_visible_set(cdetail_d->navi_item, EINA_FALSE);

	if (NULL == vcard_path)
		ct_detail_set_navi(cdetail_d, DETAIL_VIEW_MODE_INFO);
	else
		ct_detail_set_navi(cdetail_d, DETAIL_VIEW_MODE_VCARD);

	if (navi)
		return navi;
	else {
		elm_object_part_content_set(base, "elm.swallow.content", cdetail_d->navi);
		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		elm_win_resize_object_add(win, base);
		elm_win_resize_object_add(win, conform);
		elm_object_content_set(conform, base);
		evas_object_show(base);
		evas_object_show(conform);
		return conform;
	}
}

API int ctui_create_detail_view_from_phone(Evas_Object *win, Evas_Object *navi, ui_gadget_h ug, int id)
{
	PH_FN_CALL;
	PH_START_TIME_CHECK;
	ct_detail_data *cdetail_d;
	Evas_Object *content;

	cdetail_d = calloc(1, sizeof(ct_detail_data));
	p_retvm_if(!cdetail_d, -1, "cdetail_d is NULL, Not enough memory");

	cdetail_d->win = win;
	cdetail_d->navi = navi;
	cdetail_d->ug = ug;
	cdetail_d->prev_navi_item = elm_naviframe_top_item_get(navi);
	cdetail_d->person_id = id;

	content = ct_detail_create_content(cdetail_d);
	if (NULL == content) {
		ERR("ct_detail_create_content() Failed");
		free(cdetail_d);
		return -1;
	}

	cdetail_d->navi_item = elm_naviframe_item_push(cdetail_d->navi, NULL, NULL, NULL, content, NULL);
	elm_naviframe_item_title_visible_set(cdetail_d->navi_item, EINA_FALSE);

	ct_detail_set_navi(cdetail_d, DETAIL_VIEW_MODE_INFO);

	PH_END_TIME_CHECK();
	return 0;
}



