/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <vconf.h>
#include <vconf-keys.h>
#include <ui-gadget-module.h>
#include "phone.h"
#include "ct-list.h"
#include "ct-group.h"
#include "ct-detail.h"
#include "ct-list-utils.h"
#include "ct-list-view-ug.h"
#include "ct-list-contact-view.h"
#include "ct-list-edit-view.h"

API void ctui_list_view_reset(ct_list_data *clist_d)
{
	evas_object_data_del(clist_d->navi, "done_button");
	evas_object_data_del(clist_d->navi, "edit_button");
	evas_object_data_del(clist_d->navi, "new_button");
	evas_object_data_del(clist_d->navi, "view_button");
	evas_object_data_del(clist_d->navi, "account_button");

	if (CT_UG_REQUEST_CHECK != clist_d->ug_request &&
			CT_UG_REQUEST_CHECK_FOR_NUMBER != clist_d->ug_request &&
			CT_UG_REQUEST_CHECK_FOR_EMAIL != clist_d->ug_request &&
			CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL != clist_d->ug_request)
		if (CT_LIST_MODE_EDIT == clist_d->list_mode)
			ct_list_edit_mode_end(clist_d);

	if (elm_naviframe_top_item_get(clist_d->navi) != clist_d->navi_item)
		elm_naviframe_item_pop_to(clist_d->navi_item);
	else
		ctui_list_set_unsweep(clist_d);

	if (clist_d->child_ug) {
		ug_destroy(clist_d->child_ug);
		clist_d->child_ug = NULL;
	}
}

API void ctui_list_view_language_changed(Evas_Object *navi)
{
	PH_FN_CALL;
	ct_list_data *clist_d;
	void *cdetail_d;
	clist_d = evas_object_data_get(navi, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");
	ctui_list_genlist_update(clist_d);
	if (clist_d->searchbar)
		elm_object_part_text_set(clist_d->searchbar, "elm.guidetext", S_(CT_SYS_BODY_SEARCH));

	ctui_list_update_selection_info(clist_d);

	cdetail_d = evas_object_data_get(navi, "cdetail_d");
	if (cdetail_d)
		ct_detail_view_language_changed(cdetail_d);
}

static int create_list_view(Evas_Object *base, ct_list_data *clist_d)
{
	char buf[PH_TEXT_SHORT_LEN];

	if (NULL == clist_d->navi) {
		p_retvm_if(NULL == base, FALSE, "base is NULL");
		clist_d->navi = elm_naviframe_add(base);
		p_retvm_if(NULL == clist_d->navi, FALSE, "navi is NULL");
		elm_naviframe_prev_btn_auto_pushed_set(clist_d->navi, EINA_FALSE);
		elm_object_part_content_set(base, "elm.swallow.content", clist_d->navi);
		evas_object_show(clist_d->navi);
	}
	ct_list_init_account_data(clist_d);
	if (CT_GET_LIST_VCARD != clist_d->list_op)
		ct_list_get_contacts(clist_d, &clist_d->list);
	clist_d->base_layout = ct_list_create_contact_view(clist_d);

	FREEandSTRDUP(clist_d->title, T_(CT_GET_TEXT_BASIC, CTTEXT_ALL_CONTACTS));
	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);
	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);

	return TRUE;
}

static Evas_Object* list_create_vcard_list_view(Evas_Object *base, ct_list_data *clist_d)
{
	char buf[PH_TEXT_SHORT_LEN];

	p_retvm_if(NULL == base, FALSE, "base is NULL");

	clist_d->navi = elm_naviframe_add(base);
	p_retvm_if(NULL == clist_d->navi, FALSE, "navi is NULL");
	elm_naviframe_prev_btn_auto_pushed_set(clist_d->navi, EINA_FALSE);
	evas_object_show(clist_d->navi);

	clist_d->base_layout = ct_list_create_contact_view(clist_d);

	clist_d->title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_VCARD_CONTACTS));
	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);
	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);
	if (clist_d->select_result_cb)
		ctui_list_edit_mode_start(clist_d);
	else
		ctui_list_navi_set(CT_LIST_MODE_DEFAULT, clist_d);

	elm_object_part_content_set(base, "elm.swallow.content", clist_d->navi);

	return clist_d->navi;
}

static int __list_parse_bundle_by_operation(service_h service, ct_list_data* clist_d)
{
	char* val = NULL;
	char* val1 = NULL;
	char *operation = NULL;

	service_get_operation(service, &operation);
	p_retvm_if(NULL == operation, 0, "operation is NULL");
	PH_DBG("operaion = %s", operation);

	if(strcmp(operation, CT_OPERATION_SOCIAL_CHOOSE) == 0) {
		service_get_extra_data(service, CT_OPERATION_ITEM_TYPE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_ITEM_TYPE_PERSON) != 0) {
				ERR("CT_OPERATION_ITEM_TYPE_PERSON not matched %s", val);
				free(val);
				return -1;
			}
			free(val);
			val = NULL;
		}
		else {
			ERR("CT_OPERATION_ITEM_TYPE is mandatory");
			return -1;
		}
		clist_d->service_operation = service;
		clist_d->ug_request = CT_UG_REQUEST_UPDATE_CONTACT;
		clist_d->list_op = CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID;

		service_get_extra_data(service, CT_OPERATION_PHONE, &val1);
		if(val1) {
			clist_d->ug_request = CT_UG_REQUEST_UPDATE_WITH_NUM;
			clist_d->ug_data = val1;
			return 1;
		}

		service_get_extra_data(service, CT_OPERATION_EMAIL, &val1);
		if(val1) {
			clist_d->ug_request = CT_UG_REQUEST_UPDATE_WITH_EMAIL;
			clist_d->ug_data = val1;
			return 1;
		}

		service_get_extra_data(service, CT_OPERATION_URL, &val1);
		if(val1) {
			clist_d->ug_request = CT_UG_REQUEST_UPDATE_WITH_WEB;
			clist_d->ug_data = val1;
			return 1;
		}
	}
	else if(strcmp(operation, CT_OPERATION_SOCIAL_PICK) == 0) {
		service_get_extra_data(service, CT_OPERATION_ITEM_TYPE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_ITEM_TYPE_PERSON) != 0) {
				ERR("CT_OPERATION_ITEM_TYPE_PERSON not matched %s", val);
				free(val);
				return -1;
			}
			free(val);
			val = NULL;
		}
		else {
			ERR("CT_OPERATION_ITEM_TYPE is mandatory");
			return -1;
		}

		service_get_extra_data(service, CT_OPERATION_SELECTION_MODE, &val);
		if(val) {
			if(strcmp(val, CT_OPERATION_SELECTION_MODE_SINGLE) == 0) {
				service_get_extra_data(service, CT_OPERATION_RESULT_TYPE, &val1);
				if(val1) {
					if(strcmp(val1, CT_OPERATION_RESULT_TYPE_PHONE) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_SELECT_NUMBER;
						clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER;
					}
					else if(strcmp(val1, CT_OPERATION_RESULT_TYPE_EMAIL) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_SELECT_EMAIL;
						clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_EMAIL;
					}
					else if(strcmp(val1, CT_OPERATION_RESULT_TYPE_ITEM_ID) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_SELECT_PERSON;
					}
					else if(strcmp(val1, CT_OPERATION_RESULT_TYPE_VCARD) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_SELECT_VCARD;

					}
					else {
						ERR("CT_OPERATION_RESULT_TYPE is wrong value %s", val1);
						free(val);
						free(val1);
						return -1;
					}
					free(val1);
					clist_d->service_operation = service;
				}
				else {
					ERR("CT_OPERATION_RESULT_TYPE is wrong value %s", val1);
					free(val);
					free(val1);
					return -1;
				}
			}
			else if(strcmp(val, CT_OPERATION_SELECTION_MODE_MULTIPLE) == 0) {
				service_get_extra_data(service, CT_OPERATION_RESULT_TYPE, &val1);
				if(val1) {
					if(strcmp(val1, CT_OPERATION_RESULT_TYPE_PHONE) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_CHECK_FOR_NUMBER;
						clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER;
					}
					else if(strcmp(val1, CT_OPERATION_RESULT_TYPE_EMAIL) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_CHECK_FOR_EMAIL;
						clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_EMAIL;
					}
					else if(strcmp(val1, CT_OPERATION_RESULT_TYPE_ITEM_ID) == 0) {
						clist_d->ug_request = CT_UG_REQUEST_CHECK;
					}
					else {
						ERR("CT_OPERATION_RESULT_TYPE is wrong value %s", val1);
						free(val);
						free(val1);
						return -1;
					}
					free(val1);
					clist_d->service_operation = service;
				}
				else {
					ERR("CT_OPERATION_RESULT_TYPE is wrong value %s", val1);
					free(val);
					free(val1);
					return -1;
				}
			}
			else {
				ERR("CT_OPERATION_SELECTION_MODE is wrong %s", val);
				free(val);
				return -1;
			}
			free(val);
		}
		else {
			ERR("CT_OPERATION_SELECTION_MODE is mandatory");
			return -1;
		}
	}
	else {
		ERR("operation wrong");
		return -1;
	}

	return 1;
}

static void *list_on_create(ui_gadget_h ug, enum ug_mode mode,
		service_h service, void *priv)
{
	int ret;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	Evas_Object *bg, *base;
	Evas_Object *parent;
	ct_list_data *clist_d = priv;
	char *operation = NULL;

	p_retvm_if(NULL == ug || NULL == priv, NULL, "The parameter is invalid(ug=%p, priv=%p)", ug, priv);

	clist_d->win = ug_get_window();
	parent = ug_get_parent_layout(ug);
	p_retvm_if(NULL == parent, NULL, "ug_get_parent_layout() return NULL");
	clist_d->ug = ug;

	bindtextdomain(PACKAGE, LOCALEDIR);

	err = contacts_connect2();
	p_retvm_if(CONTACTS_ERROR_NONE != err, NULL, "contacts_connect2() Failed(%d)", err);

	service_get_operation(service, &operation);
	if(operation) {
		ret = __list_parse_bundle_by_operation(service, clist_d);
		free(operation);
		if(ret < 0) {
			ERR("__list_parse_bundle_by_operation invalid parameter");
			ret = ct_list_parse_bundle(service, clist_d);
		}
	}
	else {
		ret = ct_list_parse_bundle(service, clist_d);
	}

	if (UG_MODE_FULLVIEW == mode)
		base = phone_create_base_layout(parent, true);
	else
		base = phone_create_base_layout(parent, false);

	clist_d->base_ab = -1;

	if (-2 == ret) {
		Evas_Object *popup;
		popup = elm_popup_add(base);
		evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		elm_object_text_set(popup, S_(CT_SYS_POP_FILE_NOT_FOUND));
		elm_popup_timeout_set(popup, 1.5);
		evas_object_show(popup);
		evas_object_smart_callback_add(popup, "timeout", ctui_list_ug_hide_cb, clist_d->ug);
		return base;
	}
	p_retvm_if(-1 == ret, NULL, "ct_list_parse_bundle() Failed");

	bg = phone_create_bg(base);
	if (NULL == bg) {
		ERR("phone_create_bg() return NULL");
		evas_object_del(base);
		err = contacts_disconnect2();
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
		return NULL;
	}

	if (clist_d->ug_request == CT_UG_REQUEST_SELECT_GROUP ||
			clist_d->ug_request == CT_UG_REQUEST_SELECT_GROUP_ADD_TO_HOME) {
		Evas_Object *layout = phone_create_group_view_ug(clist_d->ug, base, clist_d->ug_request);
		phone_group_set_navi(layout);
	}
	else if (FALSE == create_list_view(base, clist_d)) {
		evas_object_del(base);
		err = contacts_disconnect2();
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
	}
	else {
		if (CT_UG_REQUEST_CHECK == clist_d->ug_request ||
				CT_UG_REQUEST_CHECK_FOR_NUMBER == clist_d->ug_request ||
				CT_UG_REQUEST_CHECK_FOR_EMAIL == clist_d->ug_request ||
				CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == clist_d->ug_request)
			ctui_list_edit_mode_start(clist_d);
		else
			ctui_list_navi_set(CT_LIST_MODE_DEFAULT, clist_d);
	}

	return base;
}

static void list_on_message(ui_gadget_h ug, service_h msg, service_h service, void *priv)
{
}

static void list_on_pause(ui_gadget_h ug, service_h service, void *priv)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_list_data *clist_d = priv;
	err = contacts_setting_get_name_display_order(&(clist_d->order_display));
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_setting_get_name_display_order() Failed(%d)", err);
	clist_d->is_background = true;
}

static void list_on_resume(ui_gadget_h ug, service_h service, void *priv)
{
	ct_list_data *clist_d = priv;
	ct_detail_data *cdetail_d;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e order_display;

	clist_d->is_background = false;

	err = contacts_setting_get_name_display_order(&order_display);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_setting_get_name_display_order() Failed(%d)", err);

	if (clist_d->order_display != order_display) {
		ctui_list_genlist_update(clist_d);
		cdetail_d = evas_object_data_get(clist_d->navi, "cdetail_d");
		if (cdetail_d)
			ctui_detail_genlist_update(cdetail_d);
	}
	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
		ctui_list_genlist_update(clist_d);
	}
}

static void list_on_event(ui_gadget_h ug, enum ug_event event, service_h service, void *priv)
{
	ct_list_data *clist_d = priv;

	switch (event) {
	case UG_EVENT_ROTATE_PORTRAIT:
	case UG_EVENT_ROTATE_PORTRAIT_UPSIDEDOWN:
		break;
	case UG_EVENT_ROTATE_LANDSCAPE:
	case UG_EVENT_ROTATE_LANDSCAPE_UPSIDEDOWN:
		break;
	case UG_EVENT_LANG_CHANGE:
		ctui_list_view_language_changed(clist_d->navi);
		break;
	default:
		break;
	}
}

static void list_on_destroy(ui_gadget_h ug, service_h service, void *priv)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_list_data *clist_d =priv;
	p_retm_if(NULL == priv, "The parameter is invalid(priv=%p)", priv);
	p_retm_if(NULL == ug, "The parameter is invalid(ug=%p)", ug);

	free(clist_d->ug_data);

	if(clist_d->ug) {
		evas_object_del(ug_get_layout(ug));
	}

	err = contacts_disconnect2();
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_disconnect2() Failed(%d)", err);
}

API int UG_MODULE_INIT(struct ug_module_ops *ops)
{
	ct_list_data *clist_d;

	p_retvm_if(!ops, -1, "ops is NULL");

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, -1, "UG data is NULL, Not enough memory");

	ops->create = list_on_create;
	ops->start = NULL;
	ops->pause = list_on_pause;
	ops->resume = list_on_resume;
	ops->destroy = list_on_destroy;
	ops->message = list_on_message;
	ops->event = list_on_event;
	ops->key_event = NULL;
	ops->priv = clist_d;
	ops->opt = UG_OPT_INDICATOR_ENABLE;

	return 0;
}

API void UG_MODULE_EXIT(struct ug_module_ops *ops)
{
	p_retm_if(!ops, "ops is NULL");

	free(ops->priv);
	ops->priv = NULL;
}

// Setting - export,import / Group - Message,Email,Share,ChatOn
API Evas_Object* ctui_create_list_select_view(Evas_Object *navi,
		ct_select_result_cb result_cb, void *result_cb_data, int list_op, int id_ab, int id_grp, int cnt_max)
{
	PH_DBG("list_op = %d, id_ab = %d, id_grp = %d", list_op, id_ab, id_grp);

	ct_list_data *clist_d;

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, NULL, "clist_d is NULL, Not enough memory");

	clist_d->navi = navi;
	clist_d->select_result_cb = result_cb;
	clist_d->select_result_cb_data = result_cb_data;
	clist_d->list_op = list_op;
	clist_d->base_ab = id_ab;
	clist_d->base_grp = id_grp;

	if(cnt_max > 0)
	{
		clist_d->cnt_max_check = cnt_max;
	}

	switch(list_op) {
	case CT_GET_LIST_ALL_PERSON_HAD_NUMBER:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OF_GROUP_ID:
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OF_ADDRBOOK_ID:
		clist_d->ug_request = CT_UG_REQUEST_CHECK_FOR_NUMBER;
		break;
	case CT_GET_LIST_ALL_PERSON_HAD_EMAIL:
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_HAD_EMAIL_OF_GROUP_ID:
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_EMAIL_OF_ADDRBOOK_ID:
		clist_d->ug_request = CT_UG_REQUEST_CHECK_FOR_EMAIL;
		break;
	case CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID:
	case CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_GROUP_ID:
	case CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID:
		clist_d->ug_request = CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL;
		break;
	default:
		clist_d->ug_request = CT_UG_REQUEST_CHECK;
		break;
	}

	ct_list_get_contacts(clist_d, &clist_d->list);
	clist_d->base_layout = ct_list_create_contact_view(clist_d);

	ct_list_get_title(clist_d, list_op, id_ab, id_grp);

	if (clist_d->sub_title) {
		clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, clist_d->title, NULL,
				NULL, clist_d->base_layout, NULL);
		elm_object_item_part_text_set(clist_d->navi_item, "subtitle", clist_d->sub_title);
	}
	else
		clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, clist_d->title, NULL,
				NULL, clist_d->base_layout, NULL);
	ctui_list_edit_mode_start(clist_d);

	return clist_d->navi;
}

API void ctui_create_favorite_list_select_view(Evas_Object *win, Evas_Object *navi, Eina_List *list)
{
	ct_list_data *clist_d;
	char buf[PH_TEXT_SHORT_LEN];
	clist_d = calloc(1, sizeof(ct_list_data));
	p_retm_if(!clist_d, "clist_d is NULL, Not enough memory");

	clist_d->win = win;
	clist_d->navi = navi;
	clist_d->contact_add_cb = ct_list_add_contacts_to_favorite;
	clist_d->ug_request = CT_UG_REQUEST_CHECK;
	clist_d->list = list;
	clist_d->base_layout = ct_list_create_contact_view(clist_d);
	clist_d->title = strdup(T_(CT_GET_TEXT_BASE_OP, CTTEXT_ADD_TO_FAVOURITES));
	clist_d->cnt_list_total = eina_list_count(list);
	clist_d->base_ab = -1;

	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);

	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);
	ctui_list_edit_mode_start(clist_d);
	evas_object_data_set(clist_d->navi, "favor_list_d", clist_d);
}

API Evas_Object* ctui_create_gadd_list_select_view(Evas_Object *win, Evas_Object *navi, int group_id, int ab_id, Eina_List *list)
{
	PH_FN_CALL;
	ct_list_data *clist_d;
	char buf[PH_TEXT_SHORT_LEN];

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, NULL, "clist_d is NULL, Not enough memory");

	clist_d->win = win;
	clist_d->navi = navi;
	clist_d->contact_add_cb = ct_list_add_contacts_to_group;
	clist_d->view_id = CT_LIST_MULTI_ADD_MEMBER;
	clist_d->list_op = CT_GET_LIST_PERSON_OF_ADDRBOOK_ID;
	clist_d->ug_request = CT_UG_REQUEST_CHECK;
	clist_d->base_grp = group_id;
	clist_d->base_ab = ab_id;
	clist_d->list_grp_person = list;
	ct_list_get_contacts(clist_d, &clist_d->list);
	clist_d->base_layout = ct_list_create_contact_view(clist_d);
	clist_d->title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_SELECT_CONTACTS));
	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);

	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);
	ctui_list_edit_mode_start(clist_d);
	evas_object_data_set(clist_d->navi, "gadd_list_d", clist_d);

	return clist_d->navi;
}

// Show vcard contact list check view to importing contacts from vcf file
API Evas_Object* ctui_create_vcard_list_select_view(Evas_Object *navi, Eina_List *list, const char *path,
		ct_select_result_cb result_cb, void *result_cb_data)
{
	ct_list_data *clist_d;
	char buf[PH_TEXT_SHORT_LEN];

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, NULL, "clist_d is NULL, Not enough memory");

	clist_d->navi = navi;
	clist_d->list = list;
	clist_d->cnt_list_total = eina_list_count(list);
	clist_d->vcard_path = path;
	clist_d->select_result_cb = result_cb;
	clist_d->select_result_cb_data = result_cb_data;
	clist_d->list_op = CT_GET_LIST_VCARD;
	clist_d->ug_request = CT_UG_REQUEST_CHECK;
	clist_d->base_ab = -1;

	clist_d->base_layout = ct_list_create_contact_view(clist_d);
	clist_d->title = strdup(T_(CT_GET_TEXT_BASIC, CTTEXT_VCARD_CONTACTS));
	snprintf(buf, sizeof(buf), "%s (%d)", clist_d->title, clist_d->cnt_list_total);

	clist_d->navi_item = elm_naviframe_item_push(clist_d->navi, buf, NULL, NULL, clist_d->base_layout, NULL);
	ctui_list_edit_mode_start(clist_d);

	evas_object_data_set(clist_d->navi, "clist_d", clist_d);
	return clist_d->navi;
}

// Show contact list view from vcf file
API Evas_Object* ctui_create_vcard_list_view(Evas_Object *win, Eina_List *list, const char *path)
{
	ct_list_data *clist_d;
	Evas_Object *bg, *base, *conform, *navi;

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, NULL, "clist_d is NULL, Not enough memory");

	clist_d->win = win;
	clist_d->list = list;
	clist_d->cnt_list_total = eina_list_count(list);
	clist_d->vcard_path = path;
	clist_d->base_ab = -1;

	conform = elm_conformant_add(win);
	p_retvm_if(NULL == conform, NULL, "elm_conformant_add() Failed");
	evas_object_show(conform);

	base = phone_create_base_layout(win, true);
	if (base) {
		clist_d->list_op = CT_GET_LIST_VCARD;
		navi = list_create_vcard_list_view(base, clist_d);

		evas_object_data_set(win, "naviframe", (void *)navi);

		bg = phone_create_bg(base);
		if (NULL == bg) {
			ERR("phone_create_bg() return NULL");
			evas_object_del(base);
			free(clist_d);
			return NULL;
		}

		evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		evas_object_size_hint_weight_set(conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
		elm_win_resize_object_add(win, base);
		elm_win_resize_object_add(win, conform);
		elm_object_content_set(conform, base);
		evas_object_show(base);
		evas_object_show(conform);
		evas_object_data_set(clist_d->navi, "clist_d", clist_d);

		return base;
	}
	else {
		free(clist_d);
		return NULL;
	}
}

API Evas_Object* ctui_create_phone_list_view(ph_view_info *view_info)
{
	ct_list_data *clist_d;

	clist_d = calloc(1, sizeof(ct_list_data));
	p_retvm_if(!clist_d, NULL, "clist_d is NULL, Not enough memory");

	clist_d->ug = view_info->ug;
	clist_d->win = view_info->win;
	clist_d->navi = view_info->navi;
	clist_d->tab_mode = CT_LIST_IN_MULTI_TAB;
	clist_d->navi_item = elm_naviframe_bottom_item_get(view_info->navi);

	switch (view_info->ug_request) {
	case PH_UG_REQUEST_CHECK_FOR_EMAIL:
	case PH_UG_REQUEST_SELECT_EMAIL:
		clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_EMAIL;
		break;
	case PH_UG_REQUEST_CHECK_FOR_NUMBER:
	case PH_UG_REQUEST_SELECT_NUMBER:
		clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER;
		break;
	case PH_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL:
	case PH_UG_REQUEST_SELECT_NUMBER_OR_EMAIL:
		clist_d->list_op = CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL;
		break;
	default:
		clist_d->list_op = CT_GET_LIST_ALL_PERSON;
		break;
	}

	clist_d->base_ab = ctui_get_view_mode();
	if(clist_d->base_ab > -1) {
		clist_d->list_op = CT_GET_LIST_PERSON_OF_ADDRBOOK_ID;
	}
	else
		clist_d->base_ab = -1;

	clist_d->ug_request = view_info->ug_request;
	clist_d->ug_data = view_info->ug_data;
	clist_d->cnt_max_check = view_info->ug_cnt_max;

	ct_list_init_account_data(clist_d);

	ct_list_get_contacts(clist_d, &clist_d->list);
	clist_d->base_layout = ct_list_create_contact_view(clist_d);
	evas_object_data_set(clist_d->navi, "clist_d", clist_d);

	return clist_d->base_layout;
}

API void ctui_list_view_resume(Evas_Object *navi)
{
	PH_FN_CALL;
	ct_list_data *clist_d;
	ct_detail_data *cdetail_d;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_name_display_order_e order_display;

	clist_d = evas_object_data_get(navi, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	clist_d->is_background = false;

	err = contacts_setting_get_name_display_order(&order_display);
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_setting_get_name_display_order() Failed(%d)", err);

	if (clist_d->order_display != order_display)
		ctui_list_genlist_update(clist_d);

	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
		ctui_list_genlist_update(clist_d);
	}
}

API void ctui_list_set_background(Evas_Object *obj, bool is_background)
{
	ct_list_data *clist_d;
	clist_d = evas_object_data_get(obj, "clist_d");
	p_retm_if(NULL == clist_d, "log data is null");

	clist_d->is_background = is_background;
	if (clist_d->update_timer) {
		ecore_timer_del(clist_d->update_timer);
		clist_d->update_timer = NULL;
		ctui_list_lazy_update(_contacts_person._uri, clist_d);
	}
}

API void ctui_list_view_pause(Evas_Object *navi)
{
	PH_FN_CALL;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	ct_list_data *clist_d;
	ct_detail_data *cdetail_d;
	clist_d = evas_object_data_get(navi, "clist_d");
	p_retm_if(NULL == clist_d, "evas_object_data_get() return NULL");

	err = contacts_setting_get_name_display_order(&(clist_d->order_display));
	p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_setting_get_name_display_order() Failed(%d)", err);

}

