/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/


#include <ui-gadget-module.h>

#include "phone.h"
#include "ct-list.h"
#include "ct-detail.h"
#include "ct-list-utils.h"
#include "ct-list-edit-view.h"
#include "ct-list-contact-view.h"
#include "phone-common.h"


static void list_ug_nums_popup_sel(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d = evas_object_data_get(obj, "clist_d");
	int person_id = (int)evas_object_data_get(obj, "person_id");

	if (CT_UG_REQUEST_CHECK_FOR_NUMBER == clist_d->ug_request || CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == clist_d->ug_request) {
		Elm_Object_Item *item;
		ct_person_list *person_info;

		item = evas_object_data_get(obj, "item");
		p_retm_if(NULL == item, "evas_object_data_get() return NULL");

		person_info = elm_object_item_data_get(item);
		p_retm_if(NULL == person_info, "elm_object_item_data_get() return NULL");

		person_info->id_ret = (int)data;
		person_info->id_ret_type = 1;
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
		ct_list_popup_hide_cb(obj, NULL, NULL);
	}
	else if (PH_UG_REQUEST_LAUNCH_VOICECALL == clist_d->ug_request) {
		char *num_info = (char*)elm_object_item_text_get(event_info);
		char *number;
		strtok(num_info, " ");
		number = strtok(NULL, " ");
		PH_DBG("%s", number);
		ct_list_popup_hide_cb(obj, NULL, NULL);
		phone_launch_voice_call(person_id, number);
	}
	else {
		if(clist_d->service_operation)
			phone_ug_return_value(clist_d->service_operation, 0, (int)data, 0, 0);
		else
			phone_ug_return_id(clist_d->ug, person_id, (int)data, 0);
		ug_destroy_me(clist_d->ug);
	}
}

static void list_ug_emails_popup_sel(void *data, Evas_Object *obj, void *event_info)
{
	ct_list_data *clist_d = evas_object_data_get(obj, "clist_d");
	int contact_id = (int)evas_object_data_get(obj, "person_id");

	if (CT_UG_REQUEST_CHECK_FOR_EMAIL == clist_d->ug_request || CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == clist_d->ug_request) {
		Elm_Object_Item *item;
		ct_person_list *person_info;

		item = evas_object_data_get(obj, "item");
		p_retm_if(NULL == item, "evas_object_data_get() return NULL");

		person_info = elm_object_item_data_get(item);
		p_retm_if(NULL == person_info, "elm_object_item_data_get() return NULL");

		person_info->id_ret = (int)data;
		person_info->id_ret_type = 2;
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
		ct_list_popup_hide_cb(obj, NULL, NULL);
	}
	else {
		if(clist_d->service_operation)
			phone_ug_return_value(clist_d->service_operation, 0, 0, (int)data, 0);
		else
			phone_ug_return_id(clist_d->ug, contact_id, 0, (int)data);

		ug_destroy_me(clist_d->ug);
	}
}

API Evas_Object* ctui_list_create_values_list(const char *view_uri, Evas_Object *popup, Evas_Object *list, Eina_List *values, void(*cb)(void *, Evas_Object *, void *))
{
	int id;
	int val_type;
	char *email_str_p;
	const char *type_str;
	char buf[PH_TEXT_MAX_LEN];
	Eina_List *l;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	if (NULL == list) {
		list = elm_list_add(popup);
		evas_object_size_hint_max_set(list, -1, 600 * elm_config_scale_get());	// TODO : should be revise
	}

	EINA_LIST_FOREACH(values, l, record) {
		if (NULL == record)
			continue;

		if (0 == strcmp(view_uri, _contacts_email._uri)) {
			err = contacts_record_get_int(record, _contacts_email.id, &id);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

			err = contacts_record_get_int(record, _contacts_email.type, &val_type);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

			type_str = T_(CT_GET_TEXT_EMAIL_TYPE, ctui_get_email_type_index(val_type));

			err = contacts_record_get_str_p(record, _contacts_email.email, &email_str_p);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			snprintf(buf, sizeof(buf), "%s %s", type_str, email_str_p);
		}
		else {
			char num_type_str[PH_TEXT_MAX_LEN+1];

			err = contacts_record_get_int(record, _contacts_number.id, &id);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

			err = contacts_record_get_int(record, _contacts_number.type, &val_type);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

			ctui_get_number_type_str(record, val_type, num_type_str, sizeof(num_type_str));

			err = contacts_record_get_str_p(record, _contacts_number.number, &email_str_p);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);

			snprintf(buf, sizeof(buf), "%s %s", num_type_str, email_str_p);
		}
		elm_list_item_append(list, buf, NULL, NULL, cb, (void *)id);
	}

	elm_list_mode_set(list, ELM_LIST_EXPAND);
	return list;
}

static Evas_Object* list_ug_create_popup_values_list(ct_list_data *clist_d, ct_person_list *person_info)
{
	Evas_Object *popup;
	Evas_Object *list = NULL;
	Evas_Object *btn;

	popup = elm_popup_add(clist_d->navi);
	elm_object_style_set(popup, "min_liststyle");
	evas_object_size_hint_weight_set(popup, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	if (person_info->number_list)
		list = ctui_list_create_values_list(_contacts_number._uri, popup, list, person_info->number_list, list_ug_nums_popup_sel);
	if (person_info->email_list)
		list = ctui_list_create_values_list(_contacts_email._uri, popup, list, person_info->email_list, list_ug_emails_popup_sel);

	elm_object_content_set(popup, list);
	evas_object_data_set(list, "clist_d", clist_d);
	evas_object_data_set(list, "person_id", (void *)person_info->id);

	btn = elm_button_add(popup);
	elm_object_text_set(btn, S_(PH_SYS_POP_CLOSE));
	elm_object_part_content_set(popup, "button1", btn);
	elm_object_style_set(btn, "popup_button/default");
	evas_object_smart_callback_add(btn, "clicked", ct_list_popup_hide_cb, list);

	clist_d->popup = popup;
	evas_object_show(popup);
	return list;
}

static inline void list_ug_handle_select_number(ct_list_data *clist_d, ct_person_list *person_info)
{
	char *number_str_p = NULL;
	int number_type;
	int number_id;
	Eina_List *list_contact;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if(person_info->number_list == NULL)
		person_info->number_list = ctui_person_get_number_list(list_contact);

	if (person_info->number_list) {
		if (1 == eina_list_count(person_info->number_list)) {
			record = (contacts_record_h)person_info->number_list->data;
			if (PH_UG_REQUEST_LAUNCH_VOICECALL == clist_d->ug_request) {
				err = contacts_record_get_str_p(record, _contacts_number.number, &number_str_p);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_str_p() Failed(%d)", err);
				phone_launch_voice_call(person_info->id, number_str_p);
			}
			else {
				err = contacts_record_get_int(record, _contacts_number.id, &number_id);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
				if(clist_d->service_operation)
					phone_ug_return_value(clist_d->service_operation, 0, number_id, 0, 0);
				else
					phone_ug_return_id(clist_d->ug, person_info->id, number_id, 0);
				ug_destroy_me(clist_d->ug);
			}
		}
		else
			list_ug_create_popup_values_list(clist_d, person_info);
	}
	else {
		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_NUMBER), 1.5);
	}
	ctui_free_record_list(list_contact);
}

static inline void list_ug_handle_select_email(ct_list_data *clist_d, ct_person_list *person_info)
{
	int email_id;
	Eina_List *list_contact;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if(person_info->email_list == NULL)
		person_info->email_list = ctui_person_get_email_list(list_contact);
	if (person_info->email_list) {
		if (1 == eina_list_count(person_info->email_list)) {
			record = (contacts_record_h)person_info->email_list->data;
			err = contacts_record_get_int(record, _contacts_email.id, &email_id);
			p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
			if(clist_d->service_operation)
				phone_ug_return_value(clist_d->service_operation, 0, 0, email_id, 0);
			else
				phone_ug_return_id(clist_d->ug, person_info->id, 0, email_id);
			ug_destroy_me(clist_d->ug);
		}
		else
			list_ug_create_popup_values_list(clist_d, person_info);
	}
	else {
		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);
	}
	ctui_free_record_list(list_contact);
}

static inline void list_ug_handle_select_number_or_email(ct_list_data *clist_d, ct_person_list *person_info)
{
	int number_id;
	int email_id;
	int number_list_count = 0;
	int email_list_count = 0;
	Eina_List *list_contact;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if (NULL == person_info->number_list)
		person_info->number_list = ctui_person_get_number_list(list_contact);
	if (NULL == person_info->email_list)
		person_info->email_list = ctui_person_get_email_list(list_contact);

	if (person_info->number_list)
		number_list_count = eina_list_count(person_info->number_list);
	if (person_info->email_list)
		email_list_count = eina_list_count(person_info->email_list);

	if (number_list_count == 0 && email_list_count == 0) {
		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);
	}
	else if (number_list_count == 1 && email_list_count == 0) {
		record = (contacts_record_h)person_info->number_list->data;
		err = contacts_record_get_int(record, _contacts_number.id, &number_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		if(clist_d->service_operation)
			phone_ug_return_value(clist_d->service_operation, 0, number_id, 0, 0);
		else
			phone_ug_return_id(clist_d->ug, person_info->id, number_id, 0);
		ug_destroy_me(clist_d->ug);
	}
	else if (number_list_count == 0 && email_list_count == 1) {
		record = (contacts_record_h)person_info->email_list->data;
		err = contacts_record_get_int(record, _contacts_email.id, &email_id);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		if(clist_d->service_operation)
			phone_ug_return_value(clist_d->service_operation, 0, 0, email_id, 0);
		else
			phone_ug_return_id(clist_d->ug, person_info->id, 0, email_id);
		ug_destroy_me(clist_d->ug);
	}
	else {
		list_ug_create_popup_values_list(clist_d, person_info);
	}
	ctui_free_record_list(list_contact);
}

static inline void ct_list_ug_handle_select_vcard(ct_list_data *clist_d, int person_id)
{
	bool result = false;
	char vcard_path[PH_TEXT_MAX_LEN+1];

	result = ctui_list_make_vcard(person_id, vcard_path, sizeof(vcard_path));
	PH_DBG("%d, %s", person_id, vcard_path);
	if(result == true) {
		if(clist_d->service_operation)
			phone_ug_operation_return_vcard(clist_d->service_operation, person_id, vcard_path);
		else
			phone_ug_return_vcard(clist_d->ug, person_id, vcard_path);
		ug_destroy_me(clist_d->ug);
	}
	else {
		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_CONTACTS), 1.5);
	}
}

static void list_ug_img_crop_result(ui_gadget_h ug, service_h service, void *data)
{
	Eina_List *list_writable_contact;
	ct_list_data *clist_d = data;

	p_retm_if(NULL == ug || NULL == data, "ug or data is NULL");

	if (service) {
		char *normal = NULL;
		char *full = NULL;
		void *data;
		Eina_List *l;
		contacts_record_h record = NULL;
		contacts_record_h record_image = NULL;
		contacts_error_e err = CONTACTS_ERROR_NONE;

		service_get_extra_data(service, "crop_image_path", &normal);
		p_retm_if(NULL == normal, "return value is NULL.");
		service_get_extra_data(service, "image_path", &full);

		list_writable_contact = NULL;
		list_writable_contact = ctui_person_get_writable_contact_list(clist_d->selected_person_id);

		if (list_writable_contact) {
			EINA_LIST_FOREACH(list_writable_contact, l, data) {
				record = (contacts_record_h)data;
				if (NULL == record) continue;

				record_image = ctui_get_record_image(record);
				if (NULL == record_image) {
					ERR("ctui_get_record_image() Failed(%d)", err);
					break;
				}

				err = contacts_record_set_str(record_image, _contacts_image.path, normal);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

				err = contacts_db_update_record(record);
				p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_db_update_record() Failed(%d)", err);
			}
			ctui_free_record_list(list_writable_contact);
		}

		free(normal);
		free(full);
	}
}

void ct_list_ug_handle_check_for_number(Elm_Object_Item *item, ct_list_data *clist_d)
{
	ct_person_list *person_info;
	Eina_List *list_contact;
	Evas_Object *list_nums;
	Eina_Bool all_check;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	person_info = elm_object_item_data_get(item);
	p_retm_if(NULL == person_info, "elm_object_item_data_get() return NULL");

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if (NULL == person_info->number_list)
		person_info->number_list = ctui_person_get_number_list(list_contact);

	if (NULL == person_info->number_list) {
		Evas_Object *check;
		ERR("ctui_person_get_number_list() return NULL");
		ctui_free_record_list(list_contact);

		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_NUMBER), 1.5);

		check = elm_object_part_content_get(clist_d->select_all_layout, "elm.icon");
		p_retm_if(NULL == check, "elm_object_part_content_get() return NULL");

		if (clist_d->cnt_list_total == clist_d->cnt_checked)
			all_check = EINA_TRUE;
		else
			all_check = EINA_FALSE;

		elm_check_state_set(check, all_check);
		elm_genlist_item_update(item);
		return;
	}

	if (1 == eina_list_count(person_info->number_list)) {
		record = (contacts_record_h)person_info->number_list->data;
		person_info->id_ret_type = 1;
		err = contacts_record_get_int(record, _contacts_number.id, &(person_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
	}
	else {
		list_nums = list_ug_create_popup_values_list(clist_d, person_info);
		if (list_nums)
			evas_object_data_set(list_nums, "item", item);
	}
	ctui_free_record_list(list_contact);
}

void ct_list_ug_handle_check_for_email(Elm_Object_Item *item, ct_list_data *clist_d)
{
	Eina_List *list_contact;
	Evas_Object *list_emails;
	ct_person_list *person_info;
	Eina_Bool all_check;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	person_info = elm_object_item_data_get(item);
	p_retm_if(NULL == person_info, "elm_object_item_data_get() return NULL");

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if (NULL == person_info->email_list)
		person_info->email_list = ctui_person_get_email_list(list_contact);

	if (NULL == person_info->email_list) {
		Evas_Object *check;
		ERR("ctui_person_get_email_list() return NULL");
		ctui_free_record_list(list_contact);

		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);

		check = elm_object_part_content_get(clist_d->select_all_layout, "elm.icon");
		p_retm_if(NULL == check, "elm_object_part_content_get() return NULL");

		if (clist_d->cnt_list_total == clist_d->cnt_checked)
			all_check = EINA_TRUE;
		else
			all_check = EINA_FALSE;

		elm_check_state_set(check, all_check);
		elm_genlist_item_update(item);
		return;
	}

	if (1 == eina_list_count(person_info->email_list)) {
		record = (contacts_record_h)person_info->email_list->data;
		person_info->id_ret_type = 2;
		err = contacts_record_get_int(record, _contacts_email.id, &(person_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
	}
	else {
		list_emails = list_ug_create_popup_values_list(clist_d, person_info);
		if (list_emails)
			evas_object_data_set(list_emails, "item", item);
	}
	ctui_free_record_list(list_contact);
}

void ct_list_ug_handle_check_for_number_or_email(Elm_Object_Item *item, ct_list_data *clist_d)
{
	int number_list_count = 0;
	int email_list_count = 0;
	Eina_List *list_contact;
	Evas_Object *list;
	ct_person_list *person_info;
	Eina_Bool all_check;
	contacts_error_e err = CONTACTS_ERROR_NONE;
	contacts_record_h record = NULL;

	person_info = elm_object_item_data_get(item);
	p_retm_if(NULL == person_info, "elm_object_item_data_get() return NULL");

	list_contact = ctui_person_get_contact_list(person_info->id);
	p_retm_if(NULL == list_contact, "ctui_person_get_contact_list() return NULL");

	if (NULL == person_info->email_list)
		person_info->email_list = ctui_person_get_email_list(list_contact);
	if (NULL == person_info->number_list)
		person_info->number_list = ctui_person_get_number_list(list_contact);

	if (person_info->number_list)
		number_list_count = eina_list_count(person_info->number_list);
	if (person_info->email_list)
		email_list_count = eina_list_count(person_info->email_list);

	if (0 == number_list_count && 0 == email_list_count) {
		Evas_Object *check;
		ERR("ctui_person_get_email_list() return NULL");
		ctui_free_record_list(list_contact);

		phone_show_popup(clist_d->navi, T_(CT_GET_TEXT_ERR, CTTEXT_NO_EMAIL), 1.5);

		check = elm_object_part_content_get(clist_d->select_all_layout, "elm.icon");
		p_retm_if(NULL == check, "elm_object_part_content_get() return NULL");

		if (clist_d->cnt_list_total == clist_d->cnt_checked)
			all_check = EINA_TRUE;
		else
			all_check = EINA_FALSE;

		elm_check_state_set(check, all_check);
		elm_genlist_item_update(item);
		return;
	}

	if (1 == number_list_count && 0 == email_list_count) {
		record = (contacts_record_h)person_info->number_list->data;
		err = contacts_record_get_int(record, _contacts_number.id, &(person_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		person_info->id_ret_type = 1;
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
	}
	else if (0 == number_list_count && 1 == email_list_count) {
		record = (contacts_record_h)person_info->email_list->data;
		err = contacts_record_get_int(record, _contacts_email.id, &(person_info->id_ret));
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
		person_info->id_ret_type = 2;
		person_info->checked = !person_info->checked;
		ct_list_item_checked_set(item, clist_d);
		ctui_list_update_selection_info(clist_d);
	}
	else {
		list = list_ug_create_popup_values_list(clist_d, person_info);
		if (list)
			evas_object_data_set(list, "item", item);
	}
	ctui_free_record_list(list_contact);
}

static void list_destory_child_ug(ui_gadget_h ug, void *priv)
{
	ct_list_data *clist_d = priv;
	p_retm_if(NULL == ug, "ug is NULL");
	ug_destroy(ug);

	p_retm_if(NULL == priv, "priv is NULL");
	clist_d->child_ug = NULL;
	if (clist_d->ug)
		ug_destroy_me(clist_d->ug);
}

void ct_list_ug_gl_sel(void *data, Evas_Object *obj, void *event_info)
{
	contacts_error_e err = CONTACTS_ERROR_NONE;
	int ret;
	const char *err_msg;
	contacts_record_h record = NULL;
	struct ug_cbs cbs = {0};
	ct_list_data *clist_d = data;
	ct_person_list *person_info;
	ph_view_info *view_info;
	Elm_Object_Item *item = event_info;
	Evas_Object *popup;

	PH_DBG("##### [TSP_TEST] list->detail start #####");

	elm_genlist_item_selected_set(item, EINA_FALSE);

	if (CT_GET_LIST_VCARD == clist_d->list_op && CT_UG_REQUEST_CHECK != clist_d->ug_request) {
		Evas_Object *content;

		p_retm_if(NULL == clist_d->vcard_path, "vcard_path is NULL");

		person_info = elm_object_item_data_get(item);
		p_retm_if(NULL == person_info, "elm_object_item_data_get() returns NULL");

		clist_d->idx_vcard = elm_genlist_item_index_get (item);

		content = ctui_contacts_viewer(clist_d->win, clist_d->navi, clist_d->idx_vcard, clist_d->vcard_path);
		if (NULL == content) {
			ERR("ctui_contacts_viewer() return NULL");
			elm_exit();
		}
		return;
	}

	if (CT_LIST_MODE_EDIT == clist_d->list_mode) {
		person_info = elm_object_item_data_get(item);
		p_retm_if(NULL == person_info, "elm_object_item_data_get() returns NULL");


		if (person_info->checked) {
			person_info->checked = !person_info->checked;
			ct_list_item_checked_set(item, clist_d);
			ctui_list_update_selection_info(clist_d);
			return;
		}

		if (CT_UG_REQUEST_CHECK_FOR_NUMBER == clist_d->ug_request)
			ct_list_ug_handle_check_for_number(item, clist_d);
		else if (CT_UG_REQUEST_CHECK_FOR_EMAIL == clist_d->ug_request)
			ct_list_ug_handle_check_for_email(item, clist_d);
		else if (CT_UG_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL == clist_d->ug_request)
			ct_list_ug_handle_check_for_number_or_email(item, clist_d);
		else {
			person_info->checked = !person_info->checked;
			ct_list_item_checked_set(item, clist_d);
			ctui_list_update_selection_info(clist_d);
		}

		return;
	}

	person_info = elm_object_item_data_get(item);
	p_retm_if(NULL == person_info, "elm_object_item_data_get() returns NULL");

	PH_DBG("clist_d->ug_request = %d", clist_d->ug_request);
	switch (clist_d->ug_request) {
	case CT_UG_REQUEST_LIST:
	case CT_UG_REQUEST_LIST_WITH_BACK:
		ret = ctui_create_detail_view(clist_d, person_info);
		p_warn_if(ret < 0, "ctui_create_detail_view() Failed");
		break;
	case CT_UG_REQUEST_SELECT_PERSON:
		if (clist_d->ug) {
			if(clist_d->service_operation)
				phone_ug_return_value(clist_d->service_operation, person_info->id, 0, 0, 0);
			else
				phone_ug_return_id(clist_d->ug, person_info->id, 0, 0);
			ug_destroy_me(clist_d->ug);
		}
		break;
	case CT_UG_REQUEST_SELECT_NUMBER:
	case PH_UG_REQUEST_LAUNCH_VOICECALL:
	case CT_UG_REQUEST_SELECT_CONTACT_ADD_TO_HOME:
		list_ug_handle_select_number(clist_d, person_info);
		break;
	case CT_UG_REQUEST_SELECT_EMAIL:
		list_ug_handle_select_email(clist_d, person_info);
		break;
	case CT_UG_REQUEST_SELECT_NUMBER_OR_EMAIL:
		list_ug_handle_select_number_or_email(clist_d, person_info);
		break;
	case CT_UG_REQUEST_SELECT_VCARD:
		ct_list_ug_handle_select_vcard(clist_d, person_info->id);
		break;
	case CT_UG_REQUEST_SHARE_CONTACT_INFO:
		ctui_create_share_info_ug_layout(clist_d->navi, person_info->id, clist_d->ug);
		break;
	case CT_UG_REQUEST_UPDATE_CONTACT:
	case CT_UG_REQUEST_UPDATE_WITH_NUM:
	case CT_UG_REQUEST_UPDATE_WITH_EMAIL:
	case CT_UG_REQUEST_UPDATE_WITH_WEB:
		view_info = calloc(1, sizeof(ph_view_info));
		phone_set_view_info(view_info, clist_d->win, clist_d->navi, clist_d->ug, clist_d->ug_request, clist_d->cnt_max_check, clist_d->ug_data);
		ctui_list_update_with_data(view_info, clist_d, person_info->id, clist_d->base_grp, clist_d->base_ab);
		free(view_info);
		break;
	case PH_UG_REQUEST_SET_WITH_IMG:
	case CT_UG_REQUEST_SAVE_IMG:
		clist_d->selected_person_id = person_info->id;

		cbs.result_cb = list_ug_img_crop_result;
		cbs.destroy_cb = list_destory_child_ug;
		cbs.priv = clist_d;
		clist_d->child_ug = phone_image_crop_ug(clist_d->ug, clist_d->ug_data, &cbs);
		break;
	case PH_UG_REQUEST_SET_WITH_RINGTONE:
	case CT_UG_REQUEST_SAVE_RINGTONE:
		err = contacts_db_get_record(_contacts_person._uri, person_info->id, &record);
		p_retm_if(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

		err = contacts_record_set_str(record, _contacts_person.ringtone_path, clist_d->ug_data);
		p_warn_if(CONTACTS_ERROR_NONE != err, "contacts_record_set_str() Failed(%d)", err);

		err = contacts_db_update_record(record);
		if (CONTACTS_ERROR_NONE != err) {
			ERR("contacts_db_update_record() Failed(%d)", err);
			ctui_create_contacts_error_popup(clist_d->navi, err);
		}
		else {
			err_msg = S_(CT_SYS_POP_SUCCESS);
			popup = phone_show_popup(clist_d->navi, err_msg, 1.5);
			evas_object_smart_callback_add(popup, "timeout", ctui_list_ug_hide_cb, clist_d->ug);
		}
		break;
	default:
		ERR("Invalid ug_request(%d)", clist_d->ug_request);
		break;
	}
}

