/*
* Copyright 2012 Samsung Electronics Co., Ltd
*
* Licensed under the Flora License, Version 1.1 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://floralicense.org/license/
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#ifndef __CONTACTS_LIST_H__
#define __CONTACTS_LIST_H__

#include "phone-common.h"


#define CT_LIST_MAX_SEARCH_STR 1024
#define CT_LIST_ICON_SIZE 70
#define CT_LIST_ACCOUNT_CNT_MAX 3

enum {		// this order affects to set list_op in ctui_list_create_contact_view_with_info
	CT_GET_LIST_ALL_PERSON = 0,
	CT_GET_LIST_PERSON_OF_ADDRBOOK_ID = 1,
	CT_GET_LIST_PERSON_HAD_NUMBER_OF_ADDRBOOK_ID = 2,
	CT_GET_LIST_PERSON_HAD_EMAIL_OF_ADDRBOOK_ID = 3,
	CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID = 4,
	CT_GET_LIST_PERSON_OF_WRITABLE_ADDRBOOK_ID = 5,

	CT_GET_LIST_PERSON_OF_GROUP_ID = 10,
	CT_GET_LIST_PERSON_HAD_NUMBER_OF_GROUP_ID = 11,
	CT_GET_LIST_PERSON_HAD_EMAIL_OF_GROUP_ID = 12,
	CT_GET_LIST_PERSON_HAD_NUMBER_OR_EMAIL_OF_GROUP_ID = 13,

	CT_GET_LIST_PERSON_NOT_ASSIGNED_OF_ADDRBOOK_ID = 14,
	CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OF_ADDRBOOK_ID = 15,
	CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_EMAIL_OF_ADDRBOOK_ID = 16,
	CT_GET_LIST_PERSON_NOT_ASSIGNED_HAD_NUMBER_OR_EMAIL_OF_ADDRBOOK_ID = 17,

	CT_GET_LIST_ALL_PERSON_HAD_NUMBER = 20,
	CT_GET_LIST_ALL_PERSON_HAD_EMAIL = 21,
	CT_GET_LIST_ALL_PERSON_HAD_NUMBER_OR_EMAIL = 22,
	CT_GET_LIST_VCARD = 23,
};

enum {
	CT_LIST_ITEM_NO_NUMBER,
	CT_LIST_ITEM,
	CT_LIST_ITEM_SEARCH_NO_NUMBER,
	CT_LIST_ITEM_SEARCH,
	CT_LIST_ITEM_COUNT,
};

enum {
	CT_LIST_SINGLE_VIEW,
	CT_LIST_IN_MULTI_TAB,
};

enum {
	CT_LIST_MODE_DEFAULT,
	CT_LIST_MODE_EDIT,
};

typedef struct {
	int id_ab;
	int id_grp;
	int count;
	int mode;
	bool is_read_only;
	char *name;
	char *img;
	char *img_list[6];
	Eina_Bool selected;
	Elm_Object_Item *item;
}ct_group_list;

Evas_Object* ctui_create_list_select_view(Evas_Object *navi, ct_select_result_cb result_cb, void *result_cb_data, int list_op, int id_ab, int id_grp, int cnt_max);
Evas_Object* ctui_create_gadd_list_select_view(Evas_Object *win, Evas_Object *navi, int group_id, int ab_id, Eina_List *list);
void ctui_create_favorite_list_select_view(Evas_Object *win, Evas_Object *navi, Eina_List *list);
Evas_Object* ctui_create_vcard_list_select_view(Evas_Object *navi, Eina_List *list, const char *path,
		ct_select_result_cb result_cb, void *result_cb_data);
Evas_Object* ctui_create_vcard_list_view(Evas_Object *win, Eina_List *list, const char *path);
Evas_Object* ctui_create_phone_list_view(ph_view_info *view_info);
Eina_Bool ctui_list_genlist_update(void *data);
void ctui_list_new_btn_clicked_cb(void *data, Evas_Object *obj, void *event_info);
void ctui_list_navi_set(int mode, ct_list_data *clist_d);
void ctui_list_view_reset(ct_list_data *clist_d);
void ctui_list_view_resume(Evas_Object *navi);
void ctui_list_view_pause(Evas_Object *navi);
void ctui_list_view_language_changed(Evas_Object *navi);
Evas_Object* ctui_list_create_contact_view_with_info(ph_view_info *view_info, int list_op, ct_group_list *info);
void ctui_list_set_search_text(ct_list_data *clist_d, const char *text);
void ctui_list_edit_mode_start(ct_list_data *clist_d);
void ctui_list_set_selectall(ct_list_data *clist_d);
void ctui_list_update_with_data(ph_view_info *view_info, void *data, int id, int grp_id, int ab_id);
Evas_Object* ctui_list_create_values_list(const char *view_uri, Evas_Object *popup, Evas_Object *list, Eina_List *values, void(*cb)(void *, Evas_Object *, void *));

#define CT_MAX_ADDRBOOK 200
void ct_list_init_account_data(ct_list_data *clist_d);
void ct_list_get_addrbook_icon_path(ct_list_data *clist_d, int id_ab, char *dest, int dest_len);

int ct_list_get_contacts(ct_list_data *clist_d, Eina_List **list);
char* ct_list_gl_label_get(void *data, Evas_Object *obj, const char *part);
char* ct_list_count_gl_label_get(void *data, Evas_Object *obj, const char *part);
Evas_Object* ct_list_create_contact_layout(ct_list_data *clist_d);
void ctui_create_group_input_view(Evas_Object *win, Evas_Object *navi, ct_group_list *info, ct_group_input_result_cb result_cb, void *cb_data);
void ctui_list_set_background(Evas_Object *obj, bool is_background);


#endif //__CONTACTS_LIST_H__
